package hr.genetics.models;

import java.util.ArrayList;
import java.util.List;

import hr.geometrics.Line;
import hr.geometrics.Point;
import hr.geometrics.Vector;

public class SimulationModel {

	private GenericVechileModel vechileModel;
	
	private Path environment;
	
	private Point currentPosition;
	
	private int lastCheckpointIndex = -1;
	private Checkpoint lastCheckpoint = null;
	
	private Point lastChangePosition = null;
	private double drivenDistance = 0;
	
	private Line crashLine;
	
	protected List<SimulationState> replay;
	
	public SimulationModel(GenericVechileModel v) {
		this.vechileModel = v;
		this.replay = new ArrayList<>();
	}
	
	public void setPosition(Point p) {
		this.currentPosition = p;
		this.replay.add(new SimulationState(currentPosition,
				new Vector(this.getSpeed().getX(), this.getSpeed().getY())));
	}
	
	public Point getPosition() {	
		return this.currentPosition;
	}
	
	public void changePosition(Point p) {
		
		if (this.lastChangePosition == null)
			this.lastChangePosition = p;
		
		this.setPosition(p);
		this.vechileModel.checkSensors();
		this.checkCheckpoints();
		this.updateDrivenDistance();
		
	}

	public void setSpeed(Vector v) {
		this.vechileModel.setSpeed(v);
	}
	
	public Vector getSpeed() {
		return this.vechileModel.getSpeed();
	}
	
	public Vector getNewSpeed() {
		this.vechileModel.calculateNewSpeed();
		return this.getSpeed();
	}
	
	public double getAngle() {
		return this.vechileModel.getSpeed().getAngle();	
	}
	
	public void setEnvironment(Path path) {
		this.environment = path;
		
		this.resetCheckpoints();
		this.resetChoices();
		this.resetDrivenDistance();
	}
	
	public Path getEnvironment() {
		return this.environment;
	}
	
	private void resetCheckpoints() {
		this.lastCheckpoint = null;
		this.lastCheckpointIndex = -1;
	}
	
	private void resetChoices() {
		this.lastChangePosition = null;
	}
	
	private void resetDrivenDistance() {
		this.drivenDistance = 0;
		this.lastChangePosition = null;
	}

	public boolean isCrashed() {
		
		List<Line> lines = this.environment.getLines();
		for (int i = 0; i < lines.size(); ++i) {
			if (this.currentPosition.distanceFrom(lines.get(i).getDirection()) < this.vechileModel.getRadius()
				&& this.currentPosition.getX() - 1E-8 >= Math.min(lines.get(i).getX1(), lines.get(i).getX2())
				&& this.currentPosition.getX() + 1E-8 <= Math.max(lines.get(i).getX1(), lines.get(i).getX2())
			) {
				this.crashLine = lines.get(i);
				return true;
			}/**/

			
		}
		
		if (this.currentPosition.getY() >= this.environment.getHeight())
			return true;
		
		if (this.currentPosition.getY() <= 0)
			return true;
		
		if (this.currentPosition.getX() <= 0 || this.currentPosition.getX() >= this.environment.getWidth())
			return true;
		
		return false;
		
	}
	
	public boolean isFinished() {
		
		if (this.currentPosition.getY() <= 0) {
			this.updateDrivenDistance();
			return true;
		}
		
		return false;
		
	}
	
	private void updateDrivenDistance() {
		
		this.drivenDistance += this.currentPosition.distanceFrom(this.lastChangePosition);
		if (this.currentPosition.distanceFrom(this.lastChangePosition) > 1000) {
			System.out.println(this.currentPosition.distanceFrom(this.lastChangePosition));
		}
		this.lastChangePosition = this.currentPosition;
		
	}
	
	public double getDrivenDistance() {
		return drivenDistance;
	}

	private void checkCheckpoints() {
		
		List<Checkpoint> checkpoints = this.environment.getCheckpoints();
		
		Checkpoint next;
		Vector checkpointMe;
		if (checkpoints.size() > this.lastCheckpointIndex+1) {
			next = checkpoints.get(this.lastCheckpointIndex+1);
			checkpointMe = new Vector(
					next.getPoint().getX()-this.getPosition().getX(),
					next.getPoint().getY()-this.getPosition().getY()
			);
			if (this.getSpeed().scalarProduct(checkpointMe) < 0
					&& this.getPosition().distanceFrom(next.getPoint()) < 50
			) {
				this.lastCheckpoint = next;
				this.lastCheckpointIndex++;
			}
		}
		
	}
	
	public int passedCheckpoints() {
		
		if (this.lastCheckpointIndex < 0)
			return 0;
		
		return this.lastCheckpointIndex+1;
		
	}

	public Checkpoint getLastCheckpoint() {
		return lastCheckpoint;
	}
	
	public GenericVechileModel getVechileModel() {
		return this.vechileModel;
	}
	
	public Line getCrashLine() {
		return this.crashLine;
	}
	
}
