package hr.servlets;

import hr.genetics.models.Nature;
import hr.genetics.models.Path;
import hr.genetics.models.Vehicle;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.google.gson.Gson;


public class CalculationManager extends HttpServlet {

	private static final long serialVersionUID = 1248382309967647800L;
	
	private static int populationSize = 700;
	private static int maxGenerations = 10;
	private static double mutationPossibility = 5.0E-6;

	private List<String> getQueue(HttpServletRequest req) {
		
		HttpSession session = req.getSession();
		Object queue = session.getAttribute("calculationStatus");
		if (queue == null) {
			queue = new LinkedList<>();
			session.setAttribute("calculationStatus", queue);
		}
		
		return (LinkedList<String>) queue;
		
	}
	
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		
		String action = req.getParameter("action");
		if (action == null)
			action = "getStatus";
		
		List<String> messages = getQueue(req);
		Gson gson = new Gson();
		
		HttpSession session = req.getSession();
		
		switch (action) {
			case "getStatus":
				String[] unread = messages.toArray(new String[messages.size()]);
				messages.clear();
				resp.getWriter().println(gson.toJson(unread));
			break;
			case "start":
				calculate(req, resp);
			break;
			default:
				
		}
		
	}
	
	private void calculate(HttpServletRequest req, HttpServletResponse resp) {
				
		Integer pathID = new Integer(req.getParameter("pathID"));
		
		List<String> messages = getQueue(req);
		Map<Integer, Path> paths = Path.getPaths();
		Path selectedPath = paths.get(pathID);
		
		if (selectedPath == null) {
			messages.add("Selected path not found!");
			return;
		}
		
		messages.add("Path founded!");
		
		messages.add("Calculation started!");
		
		Nature<Vehicle> nature = new Nature<>(Vehicle.class, populationSize);
		nature.logs = messages;
		nature.setEnvironment(selectedPath);
		nature.setMutationPossibility(mutationPossibility);
		nature.startEvolution(maxGenerations);
		
		messages.add("Calculation ended!");
		
	}
	
}
